/**********************************************************************/
/*                                                                    */
/*   Program Name :  timecheck                                        */
/*                                                                    */
/*      Version:  1.00  2007-03-20                                    */
/*                2.11  2008-02-19  options '--help' and '--version'  */
/*                                  were added                        */
/*                                                                    */
/*      Copyright (c) 2007-2008 T.Kondo/NICT All Right Reserved       */
/*                                                                    */
/**********************************************************************/
/*
    Display K5/VSSP or K5/VSSP32 on board time and check time label
         by T.Kondo 2007.3.19 
            Ver 1.00 2007.03.20

      How to run
          timecheck mode
               where  mode -- check mode
                              1 : check without double reading
                              2 : check with double reading

*/
#define PROGRAM_VERSION "Ver. 2008-02-19"

#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi_board.h"  // library for IP-VLBI sampler board

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory
#define TIME_READ_DOUBLE    // set TIME read double when time increment is wrong

/* set global variables */
char dev[] = DEV_FILE;  // IP-VLBI BOARD control variable
int fd_in;                 // IP-VLBI BOARD control variable
    char* dflt_logdir=K5LOGDIRDFLT;        // default log file directory
    char logdir[60];               // logfile directory
    char hostname[20];             // host name
    char cbuf[80];
int mode_double;

/* set interrupt handler */
void Handler(int n);

void SetHandler(void)
{
    if(signal(SIGINT, Handler) == SIG_ERR){
        printf("Signal handler set up failure!\n");
        abort();
    }
}

void Handler(int n)
{
     static int x=0;
     int rv;
     sprintf(cbuf,"timecheck(%s) Operator interrupt occurred !!\0");
     if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

     /* device close */

//     rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
//     if (rv == -1) {
//     perror("stop sampling failed, anyway stop ");
//     goto end;
//     }
//     else {
//        printf("Stop sampling!\n");
//     }

     //close(fd_out);

// end:
     close(fd_in);
     
     sprintf(cbuf,"timecheck(%s) ***** END ********\0",PROGRAM_VERSION);
     if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");
     
     
     exit(0);
}


int board_time_disp_and_check(void)
{
/*  
    Display board time for secmax sec
*/
    //int fd_in,rv;
    int rv;
    unsigned int time,day,sec; 
    int secold;
    int  yyyy, mm, dd;   
    int  h, m, s;     
    int  y3; // 7 bits year  (0-127)
    int counter;
    int i;


    fd_in = open(DEV_FILE, O_RDONLY); 
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
#ifdef _VSSP32
    printf("K5/VSSP32 Sampler Board Time Display and Check\n");
#else
    printf("K5/VSSP (IP-VLBI) Board Time Display and Check\n");
#endif

    if(mode_double==1){
       printf("  Mode : double reading when time seems to be wrong");
    } else {
       printf("  Mode : single reading");
    }
    printf("   (ctrl C for stop)\n\n");

    time = 0; 
    secold=90000;
    counter=0;

    //get time from device and display while
    while(1){
#ifdef _VSSP32
        rv = ioctl(fd_in, TDSIO_GET_YEAR, &y3);
        yyyy=y3+2000;
#endif
        rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
        if (rv == -1) {
            perror("ioctl() time get failed ");
            close(fd_in);
            return -1;
        }
        day=TDS_GET_DAYS(time);
        sec=TDS_GET_SEC(time);
        if (sec!=secold) {
           sec2hms(sec,&h,&m,&s);
           if(secold != 90000){
              if( sec-secold == 1 || (sec==0 && secold==86399)) {  // OK
                // sprintf(cbuf,"timecheck(%s) time icrease normaly  %d  %d\0",PROGRAM_VERSION,secold, sec);
                // printf("%s\n",cbuf);
                // if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");
              
              } else {
                  if(mode_double==1){
                     // read oncet again
#ifdef _VSSP32
                      rv = ioctl(fd_in, TDSIO_GET_YEAR, &y3);
                      yyyy=y3+2000;
#endif
                      rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
                      if (rv == -1) {
                         perror("ioctl() time get failed ");
                         close(fd_in);
                         return -1;
                      }
                      day=TDS_GET_DAYS(time);
                      sec=TDS_GET_SEC(time);
                      //                 
                      if( sec-secold == 1 || (sec==0 && secold==86399)) {  // OK
                         // sprintf(cbuf,"timecheck(%s) time icrease normaly  %d  %d\0",PROGRAM_VERSION,secold, sec);
                         // printf("%s\n",cbuf);
                         // if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");
              
                      } else {
                         sprintf(cbuf,"timecheck: time icrease abnormaly  oldsec= %d  newsec= %d\0",secold, sec);
                         printf("%s\n",cbuf);
                         if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");
                      }
                  } else {
                     sprintf(cbuf,"timecheck: time icrease abnormaly  oldsec= %d  newsec= %d\0",secold, sec);
                     printf("%s\n",cbuf);
                     if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");
                  }
              }
           }
           //printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
           //printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
#ifdef _VSSP32
           printf("%04d/%03d %02d:%02d:%02d\n\033[A",yyyy,day,h,m,s);
#else
           printf("%03d %02d:%02d:%02d\n\033[A",day,h,m,s);
#endif
           //counter++;
       }
       secold=sec;
    };
    close(fd_in);

    return 0;
}


int main(int argc, char **argv)
{
    int i;

    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          printf("\n");
          printf("timecheck %s\n",PROGRAM_VERSION);
#ifdef _VSSP32
          printf("\n  compiled for K5/VSSP32 unit\n");
#else
          printf("\n  compiled for K5/VSSP (IP-VLBI) board\n");
#endif
          printf("\nUsage: timecheck mode\n");
          printf("\n");
          printf("     where  mode ---- time read mode\n");
          printf("                      1: no double reading\n");
          printf("                      2: double reading when time seems to be abnormal\n");
          printf("\n");
          exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
          printf("\n");
          printf("timecheck %s\n",PROGRAM_VERSION);
#ifdef _VSSP32
          printf("\n  compiled for K5/VSSP32 unit\n");
#else
          printf("\n  compiled for K5/VSSP (IP-VLBI) board\n");
#endif
          printf("\n");
          exit(0);
    }
    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
    } 


    mode_double=0;
    if(argc >= 2){
       i = atoi(*(argv + 1 ));
       if( i== 2) mode_double=1;
    }

    printf("\n");
    
    /* log directory name get from enviroment variable K5LOGDIR if it is set */
    if(getenv(K5LOGDIR) != NULL){
        strcpy(logdir,getenv(K5LOGDIR));
    } else {
        strcpy(logdir,dflt_logdir);
    }

    /* get host name */
    hostget(hostname);
    
    sprintf(cbuf,"timecheck(%s) ***** START ********\0",PROGRAM_VERSION);
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");
    if(mode_double==1){
       sprintf(cbuf,"  Mode : double reading when time seems to be wrong\0");
    } else {
       sprintf(cbuf,"  Mode : single reading\0");
    }
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

    SetHandler();
    i=board_time_disp_and_check();
    
    sprintf(cbuf,"timecheck(%s) ***** END ********\0",PROGRAM_VERSION);
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

    printf("\n\n");

}
